from pathlib import Path
from os import path, mkdir
import json
import click
from cryptography.fernet import Fernet
import keyring
import os


def is_docker():
    path = '/proc/self/cgroup'
    return (
        os.path.exists('/.dockerenv') or
        os.path.isfile(path) and any('docker' in line for line in open(path))
    )


def print_pretty_json(res):
    click.echo(json.dumps(res, indent=4, sort_keys=True))


def list_by_comma(my_string):
    return [item.strip() for item in my_string.split(",")]


def error_exit(msg):
    click.echo(msg)
    exit()


def get_encryption_hash():
    secret_hash = keyring.get_password("githubsecrets", "secret_hash")
    if secret_hash:
        return secret_hash.encode()
    else:
        return None


def encrypt_data(data):
    if isinstance(data, dict):
        data = json.dumps(data)
    if not isinstance(data, str):
        error_exit("ERROR: Cannot encrypt non-strings")
    encoded = data.encode()
    fernet = Fernet(get_encryption_hash())
    return fernet.encrypt(encoded)


def decrypt_data(encrypted):
    fernet = Fernet(get_encryption_hash())
    decrypted = fernet.decrypt(encrypted)
    try:
        decrypted = json.loads(decrypted)
    except:  # noqa: E722
        pass
    return decrypted


def create_artifacts(config):
    config.raise_error = False
    artifacts = config.deserialize()['artifacts']
    current_hash = get_encryption_hash()
    if not current_hash:
        secret_hash = Fernet.generate_key().decode()
        keyring.set_password("githubsecrets", "secret_hash", secret_hash)

    for key, artifact in artifacts.items():
        if not artifact['exists']:
            if artifact['type'] == "file":
                with open(artifact['path'], 'wb') as file:
                    file.write(encrypt_data(''))
                click.echo(f"Created file - {artifact['path']}")
            elif artifact['type'] == "dir":
                mkdir(artifact['path'])
                click.echo(f"Created dir - {artifact['path']}")
            else:
                raise Exception("Unknown artifact type")
        else:
            click.echo(
                f"{artifact['type'].title()} exists - {artifact['path']}"
            )


class Config(object):
    def __init__(self, raise_error=True):
        self.verbose = False
        self.home_dir = f"{Path.home()}"
        self.ghs_dir = f"{self.home_dir}/.githubsecrets"
        self.credentials = f"{self.ghs_dir}/credentials"
        self.errors = 0
        self.errors_msg = ""
        # self.validate(raise_error)
        self.ci = False

    @staticmethod
    def get_credentials_content():
        config = Config()
        try:
            with open(config.credentials, 'rb') as file:
                encrypted = file.read()
            data = decrypt_data(encrypted)
        except json.decoder.JSONDecodeError:
            data = ''

        return data

    @staticmethod
    def set_credentials_content(credentials_content):
        config = Config()
        encrypted = encrypt_data(credentials_content)
        with open(config.credentials, 'wb') as file:
            file.write(encrypted)
        return True

    @staticmethod
    def print_response(response):
        """Prints a human-readable response"""
        res = {}
        if (response.text):
            try:
                res['body'] = response.json()
            except:  # noqa: E722
                res['body'] = response.text
        res['status_code'] = response.status_code
        print(json.dumps(res, indent=4, sort_keys=True))

    def deserialize(self):
        return {
            "artifacts": {
                "ghs_dir": {
                    "path": self.ghs_dir,
                    "type": "dir",
                    "exists": path.isdir(self.ghs_dir)
                },
                "credentials": {
                    "path": self.credentials,
                    "type": "file",
                    "exists": path.isfile(self.credentials)
                },
            },
            "errors": self.errors,
            "errors_msg": self.errors_msg
        }

    def validate(self, raise_error):
        artifacts = self.deserialize()['artifacts']
        self.errors_msg = "\nERROR: Missing files/folders\n"
        for key, artifact in artifacts.items():
            if not artifact['exists']:
                self.errors_msg += f"{artifact['path']}\n"
                self.errors += 1

        if self.errors > 0 and raise_error:
            self.errors_msg += "Fix it by executing: ghs init\n"
            error_exit(self.errors_msg)


class Validate(object):
    def __init__(self, raise_error=True):
        config = Config()
        artifacts = config.deserialize()['artifacts']
        config.errors_msg = "\nERROR: Missing files/folders\n"
        for key, artifact in artifacts.items():
            if not artifact['exists']:
                config.errors_msg += f"{artifact['path']}\n"
                config.errors += 1

        if config.errors > 0 and raise_error:
            config.errors_msg += "Fix it by executing: ghs init\n"
            error_exit(config.errors_msg)


pass_config = click.make_pass_decorator(Config, ensure=True)
pass_validate = click.make_pass_decorator(Validate, ensure=True)



import click
from .config import Config, error_exit


class Profile():
    def __init__(self, config, name):
        self.name = name
        self.credentials_content = Config.get_credentials_content()
        self.github_owner = ''
        self.personal_access_token = ''
        self.config = config
        if self.credentials_content and self.name in self.credentials_content:
            self.github_owner = self.credentials_content[self.name]['github_owner']  # noqa: E501
            self.personal_access_token = self.credentials_content[self.name]['personal_access_token']  # noqa: E501

    @staticmethod
    def lista():
        credentials_content = Config.get_credentials_content()
        msg = "\n"
        if not credentials_content:
            error_exit(
                "WARNING: Couldn't find any profile, create one by executing:\nghs profile-apply -p profile_name\n")  # noqa: E501
        for key, value in credentials_content.items():
            token_length = len(value['personal_access_token'])
            value['personal_access_token'] = \
                value['personal_access_token'][0:7] + \
                "*" * (token_length - 8)
            msg += f"profile: {key}, github_owner: {value['github_owner']}, personal_access_token: {value['personal_access_token']}\n"  # noqa: E501
        click.echo(msg)

    def apply(self, github_owner, personal_access_token):
        if self.credentials_content:
            self.credentials_content[self.name] = {
                "github_owner": github_owner,
                "personal_access_token": personal_access_token
            }
        else:
            self.credentials_content = {
                self.name: {
                    "github_owner": github_owner,
                    "personal_access_token": personal_access_token
                }
            }

        write_success = Config.set_credentials_content(
            self.credentials_content
        )
        if write_success:
            click.echo(f"SUCCESS: Applied the profile {self.name}")
        else:
            click.echo(f"FAILED: Unable to create the profile {self.name}")

    def get(self):
        pass

    def delete(self):
        if self.name in self.credentials_content:
            del self.credentials_content[self.name]
            confirm = False
            if self.config.ci:
                confirm = True
            elif click.confirm(f"Are you sure want to delete the profile {self.name} ?"):  # noqa: E501
                confirm = True

            if confirm:
                write_success = Config.set_credentials_content(
                    self.credentials_content)
                if write_success:
                    click.echo(f"SUCCESS: Deleted the profile {self.name}")
                else:
                    click.echo(
                        f"FAILED: Unable to delete the profile {self.name}")
        else:
            click.echo(f"FAILED: The profile {self.name} doesn't exist")



import click
from .config import pass_config, pass_validate, create_artifacts, \
    list_by_comma, print_pretty_json, is_docker
from .profile import Profile
from .secret import Secret


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        aliases = {
            "p": "profile",
            "s": "secret",
            "a": "apply",
            "g": "get",
            "d": "delete",
            "l": "list",
        }
        if len(cmd_name) == 2:
            words = [aliases[char] for char in cmd_name if char in aliases]
            cmd_name = "-".join(words)

        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail(f"Too many matches: {', '.join(sorted(matches))}")


# @click.group()
@ click.command(cls=AliasedGroup)
@ pass_config
@ click.option('--ci', '-ci', is_flag=True, help="Use this flag to avoid deletion confirmation prompts")  # noqa: E501
def cli(config, ci):
    """All commands can run without providing options, and then you'll be prompted to insert values.\n
Secrets' values and Personal-Access-Tokens are hidden when prompted"""  # noqa: E501
    if is_docker():
        ci = True
    config.ci = ci  # noqa: F821


@ cli.command()
@ pass_config
def init(config):
    """Create a credentials file to store your profiles"""
    create_artifacts(config)


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--profile-name', '-p', prompt=True)
@ click.option('--github-owner', '-o', prompt=True)
@ click.option(
    '--personal-access-token', '-t', prompt=True,
    hide_input=True, confirmation_prompt=True


)
def profile_apply(
    config, validate,
    profile_name, github_owner, personal_access_token
):
    """[pa] Create or modify multiple profiles providing a string delimited by commas ","\n
Example: ghs profile-apply -p 'willy, oompa'"""  # noqa: 501
    profile_names = list_by_comma(profile_name)
    for prof_name in profile_names:
        profile = Profile(config, prof_name)
        profile.apply(github_owner, personal_access_token)


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--profile-name', '-p', prompt=True)
def profile_delete(
    config, validate,
    profile_name
):
    """[pd] Delete multiple profiles providing a string delimited by commas ","\n
Example: ghs profile-delete -p 'willy, oompa'"""
    profile_names = list_by_comma(profile_name)
    for prof_name in profile_names:
        profile = Profile(config, prof_name)
        profile.delete()


@ cli.command()
@ pass_validate
@ pass_config
def profile_list(config, validate):
    """[pl] List all profile - truncates personal access tokens"""
    Profile.lista()


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--repository', '-r', prompt=True)
@ click.option('--profile-name', '-p', prompt=True)
@ click.option('--secret-name', '-s', prompt=True)
@ click.option(
    '--secret-value', '-v', prompt=True,
    hide_input=True, confirmation_prompt=True
)
def secret_apply(
    config, validate,
    repository, profile_name, secret_name, secret_value
):
    """[sa] Apply to multiple repositories providing a string delimited by commas ","\n
Example: ghs secret-apply -p willy -r 'githubsecrets, serverless-template'"""  # noqa: 501
    profile = Profile(config, profile_name)
    repositories = list_by_comma(repository)
    responses = []
    for repo in repositories:
        secret = Secret(config, profile, repo, secret_name, secret_value)
        secret.apply()
        responses.append(secret.apply())
    print_pretty_json(responses)


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--repository', '-r', prompt=True)
@ click.option('--profile-name', '-p', prompt=True)
@ click.option('--secret-name', '-s', prompt=True)
def secret_delete(
    config, validate,
    repository, profile_name, secret_name
):
    """[sd] Delete secrets from multiple repositories providing a string delimited by commas ","\n
Example: ghs secret-delete -p willy -r 'githubsecrets, serverless-template'"""  # noqa: 501
    profile = Profile(config, profile_name)
    repositories = list_by_comma(repository)
    responses = []
    for repo in repositories:
        secret = Secret(config, profile, repo, secret_name)
        responses.append(secret.delete())
    print_pretty_json(responses)


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--repository', '-r', prompt=True)
@ click.option('--profile-name', '-p', prompt=True)
@ click.option('--secret-name', '-s', prompt=True)
def secret_get(
    config, validate,
    repository, profile_name, secret_name
):
    """[sg] Get secrets from multiple repositories providing a string delimited by commas ","\n
Example: ghs secret-get -p willy -r 'githubsecrets, serverless-template'"""  # noqa: 501
    profile = Profile(config, profile_name)
    repositories = list_by_comma(repository)
    responses = []
    for repo in repositories:
        secret = Secret(config, profile, repo, secret_name)
        responses.append(secret.get())
    print_pretty_json(responses)


@ cli.command()
@ pass_validate
@ pass_config
@ click.option('--repository', '-r', prompt=True)
@ click.option('--profile-name', '-p', prompt=True)
def secret_list(
    config, validate,
    repository, profile_name
):
    """[sl] List secrets of multiple repositories providing a string delimited by commas ","\n
Example: ghs secret-delete -p willy -r 'githubsecrets, serverless-template'"""  # noqa: 501
    profile = Profile(config, profile_name)
    repositories = list_by_comma(repository)
    responses = []
    for repo in repositories:
        secret = Secret(config, profile, repo)
        responses.append(secret.lista())
    print_pretty_json(responses)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    readme = fh.read()

with open("version", "r") as fh:
    version = fh.read()

setup(
    author='Meir Gabay',
    author_email='unfor19@gmail.com',
    name='githubsecrets',
    license='MIT',
    long_description_content_type='text/markdown',
    description='A simple CLI to manage GitHub secrets, that are used with GitHub Actions',  # noqa: E501
    long_description=readme,
    version=f'{version}',
    packages=find_packages(),
    keywords='github secrets cli python',
    include_package_data=True,
    url='https://github.com/unfor19/githubsecrets',
    download_url=f'https://github.com/unfor19/githubsecrets/archive/v{version}.tar.gz',  # noqa: E501
    install_requires=[
        'Click>=7.1.1',
        'PyNaCl>=1.3.0',
        'requests>=2.23.0',
        'cryptography>=2.9',
        'keyring>=21.2.0'
    ],
    entry_points='''
        [console_scripts]
        ghs=githubsecrets.ghs:cli
    ''',
    setup_requires=[
        'setuptools>=44.1.0',
        'wheel>=0.34.2',
        'twine==3.1.1',
        'docutils>=0.16'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
)
